import { fileIsExist } from '@kevisual/use-config';
import { useFileStore } from '@kevisual/use-config/file-store';
import fs from 'fs';
import yaml from 'js-yaml';
import path from 'path';

// const payConfigPath = useFileStore('pay-config-dev.yaml')
const payConfigPath = useFileStore('pay-config.yaml');
type PayWechatConfig = {
  appid: string;
  mch_id: string;
  serial_no: string;
  api_v3_key: string;
  cert: string;
  notify_url: string;
  public_key: string;
  private_key: string;
};
type PayAlipayConfig = {
  appid: string;
  private_key: string;
  app_public_cert_content: string;
  alipay_root_cert_content: string;
  alipay_public_cert_content: string;
};
type PayConfig = {
  pay_platform: {
    pay_dev: boolean;
    wechat: PayWechatConfig;
    alipay: PayAlipayConfig;
  };
};
const readFileSync = (path: string) => {
  // return JSON.parse(fs.readFileSync(path, 'utf-8'))
  return yaml.load(fs.readFileSync(path, 'utf-8'), { schema: yaml.JSON_SCHEMA }) as PayConfig;
};
let payConfig: PayConfig;
export const getPayConfig = (): PayConfig => {
  if (payConfig) return payConfig;
  if (fileIsExist(payConfigPath)) {
    payConfig = readFileSync(payConfigPath);
    return payConfig;
  }
  payConfig = null;
  return {} as PayConfig;
};

// console.log(getPayConfig())

export const getCertPath = () => {
  const root = path.resolve(process.cwd(), 'cert');
  const ali = 'ali_test';
  return {
    alipayPublicCertPath: path.resolve(root, ali, 'alipayPublicCert.crt'),
    alipayRootCertPath: path.resolve(root, ali, 'alipayRootCert.crt'),
    appPublicCertPath: path.resolve(root, ali, 'appPublicCert.crt'),
  };
};
