import { sequelize } from '@/modules/index.ts'
import { DataTypes, Model } from 'sequelize'

export class TradeModel extends Model {
  declare id: string
  declare out_trade_no: string
  declare money: number
  declare subject: string
  declare status: string
  declare type: string
  declare data: {
    list: any[]
  }
  declare uid: string
}

TradeModel.init(
  {
    id: {
      type: DataTypes.UUID,
      primaryKey: true,
      defaultValue: DataTypes.UUIDV4
    },
    out_trade_no: {
      type: DataTypes.STRING,
      allowNull: false,
      unique: true
    },
    money: {
      type: DataTypes.INTEGER,
      allowNull: false
    },
    subject: {
      type: DataTypes.TEXT,
      allowNull: false
    },
    status: {
      type: DataTypes.STRING,
      allowNull: false,
      defaultValue: 'WAIT_BUYER_PAY' // WAIT_BUYER_PAY, TRADE_SUCCESS, TRADE_CLOSED, TRADE_FINISHED
    },
    type: {
      type: DataTypes.STRING,
      allowNull: false,
      defaultValue: 'alipay' // alipay, wechat
    },
    data: {
      type: DataTypes.JSONB,
      defaultValue: {
        list: []
      }
    },
    uid: {
      type: DataTypes.UUID,
      allowNull: true
    }
  },
  {
    sequelize,
    paranoid: true,
    modelName: 'apps_trade'
  }
)

TradeModel.sync({ alter: true, logging: false }).catch((error) => {
  console.error('sync Trade', error)
})
